// FTSMainViewController.m
// FTSMainViewController allows the user to add new tagged searches, and
// displays a a UIScrollView containing UIButtons for the tagged searches
#import "FTSMainViewController.h"
#import "FTSFlipsideViewController.h"

// constants for positioning and size of programmatically created buttons
static const int SPACING = 10;
static const int TO_EDGE = 20;
static const int SIDE = 30;

static NSString *FTS = @"FTS"; // key for storing user defaults

// private interface of class FTSMainViewController
@interface FTSMainViewController ()

// IBOutlets for programmatically interacting with GUI components
@property (weak, nonatomic) IBOutlet UIView *rootView;
@property (weak, nonatomic) IBOutlet UITextField *queryTextField;
@property (weak, nonatomic) IBOutlet UITextField *tagTextField;
@property (weak, nonatomic) IBOutlet UIScrollView *scrollView;

// tag-query pairs for Twitter searches
@property (strong, nonatomic) NSMutableDictionary *searches;

// user defaults used to store tag-query pairs on the device
@property (strong, nonatomic) NSUserDefaults *userDefaults;

// collections of UIButtons representing the searches, and
// buttons for editing, deleting and sharing searches
@property (strong, nonatomic) NSMutableArray *searchButtons;
@property (strong, nonatomic) NSMutableArray *editButtons;
@property (strong, nonatomic) NSMutableArray *deleteButtons;
@property (strong, nonatomic) NSMutableArray *shareButtons;

// URL of the search to perform when the user touches a search button
@property (strong, nonatomic) NSURL *searchURL;

// properties that store information for a search being deleted
@property int indexToDelete; // index of buttons
@property (strong, nonatomic) NSString *tagToDelete; // tag of search

@end // end private interface of class FTSMainViewController

@implementation FTSMainViewController // begins implementation of class

// display search buttons for saved searches when the view loads
- (void)viewDidLoad
{
   [super viewDidLoad]; // call superclass's viewDidLoad method
   
   // get iCloud key-value store for this app
   NSUbiquitousKeyValueStore *keyValueStore =
      [NSUbiquitousKeyValueStore defaultStore];
   
   // register to receive change notifications
   // when app's data changes in iCloud
   [[NSNotificationCenter defaultCenter]
      addObserver:self
      selector:@selector(updateSearches:)
      name:NSUbiquitousKeyValueStoreDidChangeExternallyNotification
      object:keyValueStore];   
   [keyValueStore synchronize]; // synchronize the key-value store
   
   // get the NSUserDefaults object for the app
   self.userDefaults = [NSUserDefaults standardUserDefaults];
   
   // get NSDictionary of the app's tag-query pairs
   NSDictionary *dictionary = [self.userDefaults dictionaryForKey:FTS];
   
   // if dictionary is nil, create empty NSMutableDictionary;
   // otherwise, create NSMutableCopy of dictionary
   if (dictionary == nil)
      self.searches = [[NSMutableDictionary alloc] init];
   else
      self.searches = [dictionary mutableCopy];
   
   self.searchButtons = [[NSMutableArray alloc] init]; // create array
   self.editButtons = [[NSMutableArray alloc] init]; // create array
   self.deleteButtons = [[NSMutableArray alloc] init]; // create array
   self.shareButtons = [[NSMutableArray alloc] init]; // create array
   
   // add buttons to the view for all of the previously saved searches
   for (NSString *title in self.searches)
      [self addNewButtonWithTitle:title];
} // end method viewDidLoad

// responds to NSUbiquitousKeyValueStoreDidChangeExternallyNotification;
// updates searches with any changes to the app's iCloud key-value pairs
- (void)updateSearches:(NSNotification *)notification
{
   // get the reason for the change
   NSDictionary* userInfo = [notification userInfo];
   NSNumber* reason = userInfo[NSUbiquitousKeyValueStoreChangeReasonKey];
   
   // check reason for change and update accordingly
   if (reason != nil)
   {
      // get integer value of reason
      NSInteger reasonValue = [reason integerValue];

      // if changes to the tag-query pairs occurred on another device
      if ((reasonValue == NSUbiquitousKeyValueStoreServerChange) ||
         (reasonValue == NSUbiquitousKeyValueStoreInitialSyncChange))
      {
         // get the changed keys and update searches and userDefaults
         NSArray* changedKeys =
            userInfo[NSUbiquitousKeyValueStoreChangedKeysKey];
         NSUbiquitousKeyValueStore* keyValueStore =
            [NSUbiquitousKeyValueStore defaultStore];
         
         // update searches based on iCloud changes
         for (NSString* tag in changedKeys)
         {
            NSString *query = [keyValueStore objectForKey:tag];
            
            if (query != nil)
            {
               // save the tag-query pair
               [self saveQuery:query forTag:tag syncToCloud:NO ];
            } // end if
            else
            {
               self.tagToDelete = tag; // search tag to remove
               
               for (int i = 0; i < [self.searches count]; ++i)
               {
                  NSString *label =
                     [self.searchButtons[i] titleLabel].text;
                   
                  if ([label isEqualToString:tag])
                  {
                     self.indexToDelete = i; // index of buttons to delete
                     break;
                  } // end if
               } // end for
               
               [self deleteSearch]; // remove 
            } // end else
         } // end for
      } // end if
   } // end if

   [self refreshButtonsInScrollView]; // refresh UIButtons in scrollView
} // end method updateSearches:

// add a new search button with the given title (the tag of the search)
- (void)addNewButtonWithTitle:(NSString *)title
{
   // create a new button, set title and add to searchButtons array
   UIButton *button = [UIButton buttonWithType:UIButtonTypeRoundedRect];
   [button setTitle:title forState:UIControlStateNormal];
   [self.searchButtons addObject:button]; // add to searchButtons array
   
   // tell the button to call buttonTouched: when it is touched
   [button addTarget:self action:@selector(searchButtonTouched:)
      forControlEvents:UIControlEventTouchUpInside];
      
   // sort the NSMutableArray by the UIButton's titles
   [self.searchButtons sortUsingComparator:
      ^NSComparisonResult(id button1, id button2)
      {
         return [((UIButton *)button1).titleLabel.text
            localizedCaseInsensitiveCompare:
               ((UIButton *)button2).titleLabel.text];
      }]; // end call to sortUsingComparator
   
   // do not convert autoResizing mask into constraints
   [button setTranslatesAutoresizingMaskIntoConstraints:NO];
   
   // configure search button's height with auto layout
   [button addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:
      [NSString stringWithFormat:@"V:[button(==%d)]", SIDE] options:0
      metrics:nil views:NSDictionaryOfVariableBindings(button)]];
   [self refreshButtonsInScrollView]; // refresh UIButtons in scrollView
} // end addNewButtonWithTitle:

// remove all buttons and populate scrollView with favorites
- (void)refreshButtonsInScrollView
{
   // remove all the buttons from the GUI
   for (UIView *view in self.scrollView.subviews.copy)
      [view removeFromSuperview];
   
   [self.editButtons removeAllObjects]; // clear before rebuilding GUI
   [self.deleteButtons removeAllObjects]; // clear before rebuilding GUI
   [self.shareButtons removeAllObjects]; // clear before rebuilding GUI
   
   float buttonOffset = SPACING; // reset the starting button offset
   float screenWidth; // screen width depends on orientation
   
   // determine screen width based on current status bar orientation
   if ([UIApplication sharedApplication].statusBarOrientation ==
      UIInterfaceOrientationPortrait)
      screenWidth = [[UIScreen mainScreen] bounds].size.width;
   else
      screenWidth = [[UIScreen mainScreen] bounds].size.height;
   
   // repopulate the scroll view with buttons
   for (UIButton *button in self.searchButtons)	
   {
      [self.scrollView addSubview:button]; // add button to scrollView
      
      // search button should not try to "hug" its content
      [button setContentHuggingPriority:UILayoutPriorityDefaultLow
         forAxis:UILayoutConstraintAxisHorizontal];
      
      // constraint between search button's left edge and scene left edge
      [self.rootView addConstraint:[NSLayoutConstraint
         constraintWithItem:button attribute:NSLayoutAttributeLeading
         relatedBy:NSLayoutRelationEqual toItem:self.rootView
         attribute:NSLayoutAttributeLeft multiplier:1.0 constant:TO_EDGE]];
      
      // create editButton then add it to editButtons and scrollView
      UIButton *editButton =
         [self createButtonWithIconNamed:@"UIButtonBarCompose"];
      [editButton setTranslatesAutoresizingMaskIntoConstraints:NO];
      [self.editButtons addObject:editButton]; // add to editButtons
      [self.scrollView addSubview:editButton]; // add to scrollView
      
      // make the button call editButtonTouched: when it's touched
      [editButton addTarget:self action:@selector(editButtonTouched:)
         forControlEvents:UIControlEventTouchUpInside];
      
      // create deleteButton then add it to deleteButtons and scrollView
      UIButton *deleteButton =
         [self createButtonWithIconNamed:@"UIButtonBarTrash"];
      [deleteButton setTranslatesAutoresizingMaskIntoConstraints:NO];
      [self.deleteButtons addObject:deleteButton]; // add to deleteButtons
      [self.scrollView addSubview:deleteButton]; // add to scrollView
      
      // make the button call deleteButtonTouched: when it's touched
      [deleteButton addTarget:self action:@selector(deleteButtonTouched:)
         forControlEvents:UIControlEventTouchUpInside];
      
      // create shareButton then add it to shareButtons and scrollView
      UIButton *shareButton =
         [self createButtonWithIconNamed:@"UIButtonBarAction"];
      [shareButton setTranslatesAutoresizingMaskIntoConstraints:NO];
      [self.shareButtons addObject:shareButton]; // add to shareButtons
      [self.scrollView addSubview:shareButton]; // add to scrollView
      
      // make the button call shareButtonTouched: when it's touched
      [shareButton addTarget:self action:@selector(shareButtonTouched:)
         forControlEvents:UIControlEventTouchUpInside];
  
      // add vertical auto layout constraints for buttons
      [self addVerticalConstraintForButton:button y:buttonOffset];
      [self addVerticalConstraintForButton:editButton y:buttonOffset];
      [self addVerticalConstraintForButton:deleteButton y:buttonOffset];
      [self addVerticalConstraintForButton:shareButton y:buttonOffset];

      // specify horizontal auto layout format for buttons
      NSString *horizontalFormat = [NSString stringWithFormat:
         @"[button(>=%f@900)]-%d-[editButton]-%d-[deleteButton]-%d-[shareButton]",
         screenWidth - SIDE * 3 - SPACING * 3 - TO_EDGE * 2,
         SPACING, SPACING, SPACING];
      [self.scrollView addConstraints:[NSLayoutConstraint
         constraintsWithVisualFormat:horizontalFormat
         options:0 metrics:nil views:NSDictionaryOfVariableBindings(
            button, editButton, deleteButton, shareButton)]];

      // constraint between sharebutton's right edge and scene right edge
      [self.rootView addConstraint:[NSLayoutConstraint
         constraintWithItem:shareButton
         attribute:NSLayoutAttributeTrailing
         relatedBy:NSLayoutRelationEqual toItem:self.rootView
         attribute:NSLayoutAttributeRight multiplier:1.0
         constant:-TO_EDGE]];
      
      // increase for next row of buttons in scrollView
      buttonOffset += SIDE + SPACING;
   } // end for
} // end refreshButtonsInScrollView

// creates a custom UIButton with the specified image
- (UIButton *)createButtonWithIconNamed:(NSString *)name
{
   // create new custom UIButton and set its background image
   UIButton *button = [UIButton buttonWithType:UIButtonTypeCustom];
   [button setImage:[UIImage imageNamed:name]
      forState:UIControlStateNormal];
   button.imageView.contentMode = UIViewContentModeScaleAspectFit;
   
   // add auto layout constraints to set button's width and height
   [button addConstraints:[NSLayoutConstraint
      constraintsWithVisualFormat:[NSString stringWithFormat:
         @"[button(==%d)]", SIDE]
      options:0 metrics:nil
      views:NSDictionaryOfVariableBindings(button)]];
   [button addConstraints:[NSLayoutConstraint
      constraintsWithVisualFormat:[NSString stringWithFormat:
         @"V:[button(==%d)]", SIDE]
      options:0 metrics:nil
      views:NSDictionaryOfVariableBindings(button)]];
   return button;
} // end method createButtonWithIconNamed:

// add a vertical spacing auto layout constraint to specified UIButton
- (void)addVerticalConstraintForButton:(UIButton *)button y:(int)y
{
   NSString *verticalFormat =
      [NSString stringWithFormat:@"V:|-%d-[button]", y];
   [self.scrollView addConstraints:[NSLayoutConstraint
      constraintsWithVisualFormat:verticalFormat options:0 metrics:nil
      views:NSDictionaryOfVariableBindings(button)]];
} // end method addVerticalConstraintForButton:y:

// adjust UIScrollView's contentSize after GUI is layed out
-(void)viewDidLayoutSubviews
{   
   CGSize contentSize = CGSizeMake(self.view.bounds.size.width,
      self.searchButtons.count * (SIDE + SPACING) + SPACING);
   [self.scrollView setContentSize:contentSize];
} // end method viewDidLayoutSubviews

// called when the user touches an edit button
- (void)editButtonTouched:(id)sender
{
   // get the index of the button that was touched
   int index = [self.editButtons indexOfObject:sender];
   
   // get the title of the button
   NSString *key = [self.searchButtons[index] titleLabel].text;
   self.tagTextField.text = key; 
   
   // get the search query using the button title and remove
   // percent escape sequences for display purposes
   NSString *value = 
      [self.searches[key] stringByReplacingPercentEscapesUsingEncoding:
         NSStringEnumerationByComposedCharacterSequences];
   self.queryTextField.text = value; // update queryField with the value
} // end method editButtonTouched:

// display a dialog asking whether to remove all saved searches
- (void)deleteButtonTouched:(id)sender
{
   // store tag to delete and index of buttons to delete
   self.indexToDelete = [self.deleteButtons indexOfObject:sender];
   self.tagToDelete =
      [self.searchButtons[self.indexToDelete] titleLabel].text;

   UIAlertView *alertView = [[UIAlertView alloc]
      initWithTitle:@"Are You Sure?"
      message:[NSString stringWithFormat:
         @"Are you sure you want to delete the search for tag \"%@\"?",
         self.tagToDelete]
      delegate:self cancelButtonTitle:@"Cancel"
      otherButtonTitles:@"Delete Search", nil];
   [alertView show]; // display the dialog
} // end deleteButtonTouched:

// respond to UIAlertView displayed by deleteButtonTouched:
- (void)alertView:(UIAlertView *)alertView
   clickedButtonAtIndex:(NSInteger)buttonIndex
{
   if (buttonIndex == 1) // user confirmed delete
      [self deleteSearch];
} // end method alertView:clickedButtonAtIndex:

// delete a search and corresponding buttons
- (void)deleteSearch
{
   // remove tag-query pair from searches and update userDefaults
   [self.searches removeObjectForKey:self.tagToDelete];
   [self.userDefaults setObject:self.searches forKey:FTS];
   
   // get iCloud key-value store and remove tag-query pair
   NSUbiquitousKeyValueStore* keyValueStore =
      [NSUbiquitousKeyValueStore defaultStore];
   [keyValueStore removeObjectForKey:self.tagToDelete];
   
   // remove buttons corresponding to tagToDelete
   [self.searchButtons removeObjectAtIndex:self.indexToDelete];
   [self.editButtons removeObjectAtIndex:self.indexToDelete];
   [self.shareButtons removeObjectAtIndex:self.indexToDelete];
   [self.deleteButtons removeObjectAtIndex:self.indexToDelete];
   
   [self refreshButtonsInScrollView]; // update the UIScrollView
} // end method deleteSearch

// called when the user touches a share button
- (void)shareButtonTouched:(id)sender
{
   // get the index of the button that was touched
   int index = [self.shareButtons indexOfObject:sender];
   
   // get the title of the button, then get corresponding search string
   NSString *key = [self.searchButtons[index] titleLabel].text;
   NSString *query = self.searches[key];
   
   NSString *message = @"Check out the results of this Twitter search";
   NSString *urlString = [NSString
      stringWithFormat:@"http://mobile.twitter.com/search/%@", query];
   NSArray *itemsToShare = @[message, urlString];
   
   // create a UIActivityViewController so user can choose where to share
   UIActivityViewController *activityViewController =
      [[UIActivityViewController alloc]
         initWithActivityItems:itemsToShare applicationActivities:nil];
   
   // display the UIActivityViewController
   [self presentViewController:activityViewController animated:YES
      completion:nil];
} // end method editButtonTouched:

// add or update a tag-query pair when user touches the add icon
- (IBAction)addButtonTouched:(id)sender
{
   // make the keyboard disappear
   [self.tagTextField resignFirstResponder];
   [self.queryTextField resignFirstResponder];
   
    // save the tag-query pair and sync to iCloud
   [self saveQuery:self.queryTextField.text forTag:self.tagTextField.text
      syncToCloud:YES];
   
   self.tagTextField.text = @""; // clear tagField
   self.queryTextField.text = @""; // clear queryField    
} // end method addButtonTouched:

// save a tag-query pair
-(void)saveQuery:(NSString *)query forTag:(NSString *)tag
   syncToCloud:(BOOL)sync
{
   // if neither query nor tag is empty
   if (query != nil && query.length > 0 && tag != nil && tag.length > 0)
   {
      if (self.searches[tag] == nil) // if tag doesn't exist
         [self addNewButtonWithTitle:tag]; // add a new button
      
      // create UTF8-encoded NSString representation of the search query
      NSString *encodedQuery =
         (__bridge NSString *) CFURLCreateStringByAddingPercentEscapes(
            NULL, (__bridge CFStringRef)(query), NULL,
            (__bridge CFStringRef)(@":/?#[]@!$&’()*+,;="),
            kCFStringEncodingUTF8);
      
      // add new tag-query pair to searches
      self.searches[tag] = encodedQuery;
      
      // update userDefaults
      [self.userDefaults setObject:self.searches forKey:FTS];
      [self.userDefaults synchronize]; // force immediate save to device
      
      // if sync is YES (true), add tag-query pair to iCloud
      if (sync)
      {
         NSUbiquitousKeyValueStore* keyValueStore =
            [NSUbiquitousKeyValueStore defaultStore];
         [keyValueStore setObject:query forKey:tag];
      } // end if
   } // end if
} // saveQuery:forTag: 

// load selected search in into UIWebView in FTSFlipsideViewController
- (void)searchButtonTouched:(id)sender
{
   NSString *tag = [sender titleLabel].text; // get Button's text
   NSString *query = self.searches[tag]; // get the query
   
   // format the URL
   NSString *urlString = [NSString stringWithFormat:
      @"http://mobile.twitter.com/search/%@", query];
   self.searchURL = [NSURL URLWithString:urlString]; // save current URL

   // display the FTSFlipsideViewController
   [self performSegueWithIdentifier:@"showSearchResults" sender:self];
} // end searchButtonTouched

// pass the NSURL containing the search to the flipside view for display
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
   if ([[segue identifier] isEqualToString:@"showSearchResults"])
   {
      // get the FTSFlipsideViewController and give it searchURL
      FTSFlipsideViewController *viewController =
         (FTSFlipsideViewController *) segue.destinationViewController;
      viewController.searchURL = self.searchURL; // set searchURL
      [viewController setDelegate:self]; // set flipside's delegate
   } // end if
} // end method prepareForSegue:sender:

// from FTSFlipsideViewControllerDelegate; returns to the main view when
// user touches Done button in flipside view
- (void)flipsideViewControllerDidFinish:
   (FTSFlipsideViewController *)controller
{
   [self dismissViewControllerAnimated:YES completion:nil];
} // end method flipsideViewControllerDidFinish:

@end // FTSMainViewController implementation

/**************************************************************************
 * (C) Copyright 2013 by Deitel & Associates, Inc. All Rights Reserved.   *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 *                                                                        *
 * As a user of the book, Deitel & Associates, Inc. grants you the        *
 * nonexclusive right to copy, distribute, display the code, and create   *
 * derivative apps based on the code. You must attribute the code to      *
 * Deitel & Associates, Inc. and reference the book's web page at         *
 * www.deitel.com/books/iosfp2/. If you have any questions, please email  *
 * at deitel@deitel.com.                                                  *
 *************************************************************************/
