// SpotOnViewController.m
// Controller for the Spot On Game app
#import "ViewController.h"
static const float SCALE_X = 0.25f; // end animation x scale
static const float SCALE_Y = 0.25f; // end animation y scale
static const float INITIAL_ANIMATION_DURATION = 6.0; // initial animation length
static const float SHORTEN_ANIMATION_FACTOR = 0.90; // used to increase speed
static const int INITIAL_SPOTS = 5; // initial # of spots
static const float SPOT_DELAY = 0.5f; // half second delay
static const int LIVES = 3; // start with 3 lives   
static const int MAX_LIVES = 7; // maximum # of total lives
static const int NEW_LEVEL = 10; // spots to reach new level
static const float INITIAL_SPOT_SCALE = 0.075f; // 7.5% of screen width

@implementation ViewController
//////////////////////////////////////////////////////////////////////////
// generate get and set methods for our properties
@synthesize scoreLabel;
@synthesize levelLabel;
@synthesize highScoreLabel;

// enum for detecting touched spots
enum TouchConstants { NOT_TOUCHED, TOUCHED };

// called when the view finishes loading
- (void)viewDidLoad
{
   [super viewDidLoad]; // pass the message to the superclass
   srandom(time(0)); // seed random number generation
   spots = [[NSMutableArray alloc] init]; // initialize spots
   lives = [[NSMutableArray alloc] init]; // initialize lives
   	
   // get view width and height
   CGSize size = self.view.bounds.size; // get view size
   viewWidth = size.width > size.height ? size.width : size.height; 
   viewHeight = size.width > size.height ? size.height : size.width;  

   // initialize the spot and life images
   redSpotImage = [UIImage imageNamed:@"red_spot.png"];
   greenSpotImage = [UIImage imageNamed:@"green_spot.png"];
   lifeImage = [UIImage imageNamed:@"life.png"];
   
   spotRadius = INITIAL_SPOT_SCALE * viewWidth; // calculate spot radius
   
   // get hit sound and initialize an AVAudioPlayer
   NSString *soundPath =
      [[NSBundle mainBundle] pathForResource:@"hit" ofType:@"wav"];
   NSURL *fileURL = [[NSURL alloc] initFileURLWithPath:soundPath];
   hitPlayer =
      [[AVAudioPlayer alloc] initWithContentsOfURL:fileURL error:nil];
   [hitPlayer prepareToPlay]; // preload audio so there is no delay
   
   // get miss sound and initialize an AVAudioPlayer
   soundPath =
      [[NSBundle mainBundle] pathForResource:@"miss" ofType:@"wav"];
   fileURL = [[NSURL alloc] initFileURLWithPath:soundPath];
   missPlayer =
      [[AVAudioPlayer alloc] initWithContentsOfURL:fileURL error:nil];
   [missPlayer prepareToPlay];
   
   // get disappear sound and initialize an AVAudioPlayer
   soundPath =
      [[NSBundle mainBundle] pathForResource:@"disappear" ofType:@"wav"];
   fileURL = [[NSURL alloc] initFileURLWithPath:soundPath];
   disappearPlayer =
	   [[AVAudioPlayer alloc] initWithContentsOfURL:fileURL error:nil];
   [disappearPlayer prepareToPlay];
   
   [self resetGame]; // begin a new game
} // end method viewDidLoad

// removes old objects and begins a new game
- (void)resetGame
{   
   [spots removeAllObjects]; // empty the array of spots
   animationDuration = INITIAL_ANIMATION_DURATION; // reset the draw time
   spotsTouched = 0; // reset the number of spots touched
   score = 0; // reset the score
   level = 1; // reset the level
   [scoreLabel setText:@"Score: 0"]; // reset the score label
   [levelLabel setText:@"Level: 1"]; // reset the level label
   
   // get the high score from the preferences file
   NSNumber *highScore =
      [[NSUserDefaults standardUserDefaults] valueForKey:@"highScore"];
   
   // if there isn't a current high score
   if (highScore == nil)
      highScore = [NSNumber numberWithInt:0]; // set the high score to 0
   
   // update the high score label with the high score
   [highScoreLabel setText:
      [NSString stringWithFormat:@"High score: %@", highScore]];
   gameOver = NO; // reset the gameOver boolean
   
   // add three lives
   for (int i = 0; i < LIVES; ++i)
   {
      UIImageView *life = [[UIImageView alloc] initWithImage:lifeImage];
      
      // position the views next to each other at the bottom left
      CGRect frame = CGRectMake(10 + lifeImage.size.width * i,  
         viewHeight - lifeImage.size.height - 10, 
         lifeImage.size.width, lifeImage.size.height);
      life.frame = frame; // assign the new frame
      [lives addObject:life]; // add the view to the list of lives
      [self.view addSubview:life]; // add the image to the view
   } // end for
   
   [self addNewSpot]; // add first spot

   // add remaining spots, each separated by a one-second delay
   for (int multiple = 1; multiple < INITIAL_SPOTS; ++multiple)
   {
      [self performSelector:@selector(addNewSpot) withObject:nil
         afterDelay:SPOT_DELAY * multiple]; // call addNewSpot after delay
   } // end for
} // end method resetGame

// chanes score by adding an integer
- (void)addToScore:(int)add
{
	score += add; // update the score
	
	if (score < 0)
		score = 0; // score cannot be negative
	
	// update the score label
	[scoreLabel setText:[NSString stringWithFormat:@"Score: %i", score]];
} // end method addToScore:

// adds a new spot at a random location
- (void)addNewSpot
{   
   // pick random coordinates inside the view
   float x = random() % (int)(viewWidth - 2 * spotRadius);
   float y = random() % (int)(viewHeight - 2 * spotRadius);
   
   // create a new spot
   UIImageView *spot = [[UIImageView alloc] 
      initWithImage:random() % 2 ? greenSpotImage : redSpotImage];
   
/*   // configure a gesture recognizer to handle spot taps
   UITapGestureRecognizer *spotTapRecognizer = 
      [[UITapGestureRecognizer alloc] initWithTarget:self 
         action:@selector(spotTapped:)];
   [spotTapRecognizer setNumberOfTouchesRequired:2];
   [spotTapRecognizer setDelegate:self];
   //Don't forget to set the userInteractionEnabled to YES, by default It's NO.
   spot.userInteractionEnabled = YES;
   [spot addGestureRecognizer:spotTapRecognizer];
*/   
   [spot setUserInteractionEnabled:NO];
   [spots addObject:spot]; // add the spot to the spots NSMutableArray
   [self.view addSubview:spot]; // add the spot to the main view
   
   // set the frame of variable spot with the random coordinates
   [spot setFrame:CGRectMake(x, y, spotRadius * 2, spotRadius * 2)];
   spot.tag = NOT_TOUCHED; // indicate that the spot has not been touched
   
   // delay beginning animation to give the spot time to redraw
   [self performSelector:@selector(beginSpotAnimation:) withObject:spot
              afterDelay:0.01];   
} // end method addNewSpot

// start the animation of the given spot
- (void)beginSpotAnimation:(UIImageView *)spot
{   
   // pick random coordinates inside the view
   float x = random() % (int)(viewWidth - 2 * spotRadius);
   float y = random() % (int)(viewHeight - 2 * spotRadius);
   
	[UIView animateWithDuration:animationDuration delay:0.1
      options:UIViewAnimationOptionCurveLinear | UIViewAnimationOptionAllowUserInteraction
      animations:
      ^{
       // set the ending location of the spot
       [spot setFrame:
          CGRectMake(x + spotRadius, y + spotRadius, 
             spot.image.size.width * SCALE_X, spot.image.size.height * SCALE_Y)];
      }
      
      completion:^(BOOL finished)
      {
         [self finishedAnimation:@"begin" spot:spot];
      }
   ];
} // end method beginSpotAnimation:

// event handler for a tapped spot
- (void)spotTapped:(id)sender
{

} // end method spotTapped

// method is called when the player touches the screen
- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
   BOOL hitSpot = NO; // initialize hitSpot
	NSLog(@"touchesBegan");
   
   for (UITouch *touch in touches) // loop through all the new touches
   {
      // get the location of the touch in the main view
      CGPoint point = [touch locationInView:self.view];
      
      // loop through all the spots to check if the player hit any;
      // iterate backwards so foreground spots get checked first
      for (int i = spots.count - 1; i >= 0 && !hitSpot; i--)
      {
         UIImageView *spot = [spots objectAtIndex:i];
         
         // We need to get the current location of spot, but the frame
         // of spot is already set to its ending location. To get the
         // displayed frame, we need to access the Core Animation layer.
         CGRect frame = [[spot.layer presentationLayer] frame];
         
         // compute the point at the center of the spot
         CGPoint origin = CGPointMake(frame.origin.x + frame.size.width /
            2, frame.origin.y + frame.size.height / 2);
			
         // compute the distance between the spot's center and the touch
         float distance = 
		      pow(origin.x - point.x, 2) + pow(origin.y - point.y, 2);
         distance = sqrt(distance); // square root to complete formula
         
         // if spot hasn't been touched, check if touch is within the spot
         if (spot.tag == NOT_TOUCHED && distance <= frame.size.width / 2)
         {
            spot.image = redSpotImage; // change to touched spot image
            [self touchedSpot:spot]; // call the touchedSpot: method
            hitSpot = YES; // a spot has been touched
            spot.tag = TOUCHED; // this spot has been touched  
         } // end if
      } // end for
   } // end for
   
   if (!hitSpot) // if the player missed
   {
      [missPlayer play]; // play the miss sound
      [self addToScore:-20 * level]; // remove some points from the score
   } // end if
} // end method touchesBegan:withEvent:

- (void)touchedSpot:(UIImageView *)spot
{	
   ++spotsTouched; // increment the number of spots touched
   hitPlayer.currentTime = 0; // reset the playback to the beginning
   [hitPlayer play]; // play the sound for when the player hits a spot
	[self addToScore:10 * level]; // add some points to the score
   
   // increment level if the player touched 10 spots in the current level
   if (spotsTouched % NEW_LEVEL == 0)
   {
      ++level; // increment the level
      animationDuration *= SHORTEN_ANIMATION_FACTOR; // speed up the game
      
      // update the level label
      [levelLabel setText:
         [NSString stringWithFormat:@"Level: %i", level]];

      // add a new life if maximum number of lives has not been reached
      if (lives.count < MAX_LIVES)
      {
         // load the life image
         UIImageView *life = [[UIImageView alloc] initWithImage:lifeImage];
         
         // position the View to the right of the previous life View
         CGRect frame = CGRectMake(10 + lifeImage.size.width * lives.count, 
            viewHeight - lifeImage.size.height - 10, lifeImage.size.width, 
            lifeImage.size.height);
         life.frame = frame; // set life's frame
         [self.view addSubview:life]; // add life to View
         [lives addObject:life]; // add life to lives array
      } // end if
   } // end if
   
   // stop the current animation and start a new one at the same place
//   CGRect frame = [[spot.layer presentationLayer] frame]; // get the frame
   [spot.layer removeAllAnimations]; // stop the animation
/*   spot.frame = frame; // move the spot to where the old animation ended
   [spot setNeedsDisplay]; // redraw the spot
   
   // give the spot time to redraw by delaying the end animation
   [self performSelector:@selector(beginSpotEndAnimation:) withObject:spot
				  afterDelay:0.01];
 */
   [spots removeObject:spot];  
   [spot removeFromSuperview]; // remove the old spot
   [self addNewSpot]; // add a new spot   
} // end method touchedSpot:

// starts the fade-away animation for a spot that's been touched
- (void)beginSpotEndAnimation:(UIImageView *)spot
{
	// start a new animation lasting 0.8 seconds
   [UIView animateWithDuration:0.8 delay:0
	 
      // allow the view to process touch events during the animation
      options:UIViewAnimationOptionAllowUserInteraction
      animations:^
      {
         // make the spot stay in the same place and disappear
         CGRect frame = spot.frame; // get the current frame
         frame.origin.x += frame.size.width / 2; // set x to the center
         frame.origin.y += frame.size.height / 2; // set y to the center
         frame.size.width = 0; // set the width to 0
         frame.size.height = 0; // set the height to 0
         [spot setFrame:frame]; // assign the new frame
         [spot setAlpha:0.0]; // set the spot to be fully transparent
      }
      
      completion:^(BOOL finished)
      {
         // when the animation finishes, call the finishedAnimation method
         [self finishedAnimation:@"end" spot:spot];
      }
    ];
} // end method beginSpotEndAnimation:

// method is automatically called when an animation ends
- (void)finishedAnimation:(NSString *)animationId spot:(UIImageView *)spot
{	
	// if the game has already been lost, return
	if (gameOver)
		return;
   
	// if it was an ending animation that finished, add a new spot
	if (animationId == @"end")
	{
		// remove spot from the spots NSMutableArray
		[spots removeObject:spot];  
		[spot removeFromSuperview]; // remove the old spot
		[self addNewSpot]; // add a new spot
	} // end if
	else if (spot.tag == NOT_TOUCHED) // a spot was missed
	{
		[disappearPlayer play]; // play disappearing spot sound effect
		
		// the game has been lost
		if (lives.count == 0)
			[self gameOver];
		else // remove one life
		{
			UIImageView *life = [lives lastObject]; // get the last life
			[lives removeLastObject]; // remove the life from the array
			[life removeFromSuperview]; // remove the life from the screen
			
			// remove the old spot and create a new one
			[spot removeFromSuperview]; // remove old spot
			[self addNewSpot]; // add a new spot
		} // end else
	} // end else
} // end method finishedAnimation:finished:context:

// called when all the lives have been lost
- (void)gameOver
{
	for (UIImageView *spot in spots) // delete all old spots
	{
		[spot removeFromSuperview]; // remove the spot from the View
		[spot.layer removeAllAnimations]; // stop existing animations
	} // end for
	
	gameOver = YES; // the game is over
	
	// display the game-over alert
	NSString *message =
	[NSString stringWithFormat:@"Score: %i", score];
	
	// get the standard user defaults
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	
	// get the current high score
	int highScore = [defaults integerForKey:@"highScore"];
	
	// if the score of the last game is greater than the high score
	if (score > highScore)
		
		// update the file with the new high score
		[defaults setInteger:score forKey:@"highScore"];
	
	UIAlertView *alert = [[UIAlertView alloc]
                         initWithTitle:@"Game Over" message:message delegate:self
                         cancelButtonTitle:@"Reset Game" otherButtonTitles:nil];
	[alert show]; // show the alert
 }

// called when the player touches the "Reset Game" button
- (void)alertView:(UIAlertView *)alertView 
   clickedButtonAtIndex:(NSInteger)buttonIndex
{
	[self resetGame]; // create a new game
} // end method alertView:clickedButtonAtIndex:

// called to determine what orientations our View allows
- (BOOL)shouldAutorotateToInterfaceOrientation:
   (UIInterfaceOrientation)interfaceOrientation
{
	// allow only the portrait orientation
   return (interfaceOrientation == UIInterfaceOrientationLandscapeLeft || 
      interfaceOrientation == UIInterfaceOrientationLandscapeRight);
} // end method shouldAutorotateToInterfaceOrientation:
@end

/**************************************************************************
 * (C) Copyright 2010 by Deitel & Associates, Inc. All Rights Reserved.   *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 *                                                                        *
 * As a user of the book, Deitel & Associates, Inc. grants you the        *
 * nonexclusive right to copy, distribute, display the code, and create   *
 * derivative apps based on the code for noncommercial purposes only--so  *
 * long as you attribute the code to Deitel & Associates, Inc. and        *
 * reference www.deitel.com/books/iPhoneFP/. If you have any questions,   *
 * or specifically would like to use our code for commercial purposes,    *
 * contact deitel@deitel.com.                                             *
 *************************************************************************/

